// C Header File

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Title:        AGENDA V4.2  oooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Author:       Adrian Bader o - o A.Bader@gmx.ch ooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
// -- Best viewed with TAB-STOP 4! ------------------------------------------
// --------------------------------------------------------------------------


#ifndef __AGENDA__
#define	__AGENDA__

#include "Date.h"

// --------------------------------------------------------------------------
// --- Structures -----------------------------------------------------------
// --------------------------------------------------------------------------

extern BITMAP 			AgendaIcon;
extern BITMAP			IconLogo;
extern BITMAP			AddressBookIcon;
extern BITMAP			NotesIcon;
extern BITMAP			InetLinksIcon;

extern BITMAP			MoreData;
extern BITMAP			Cross;
extern BITMAP			BirthDay;
extern BITMAP			smallCross;
extern BITMAP			smallBirthDay;

extern int				mnss[];
extern int				mnsn[];

typedef enum {OFF, ON} TOnOff;

typedef struct
{
	unsigned int	Size;
	unsigned int	Version;			// Higher byte always 0xFF,to identify first version, which didn't have a versioninfo
	unsigned int	Flags;
	unsigned long	SpecialDays;
	char			Password[10];
	unsigned int	nStarts;			// num starts since last backup
	unsigned int	LastTask;			// last used task: Agenda || Addressbook || Notepad || Weblinks

	unsigned int	LastAgendaDay;		// last viewed day
	unsigned int	LastAgendaEditPos;	// cursor position of dayplan editor
	unsigned int	LastAgendaMode;		// view mode: Month view, normal view

	unsigned int	LastABGroup;		// group on which the cursor was
	unsigned int	LastABEntry;		// entry on which the cursor was
	unsigned int	LastABEditPos;		// last edited addressbook string
	unsigned int	LastABMode;			// not yet implemented

	unsigned int	LastNPGroup;
	unsigned int	LastNPEntry;
	unsigned int	LastNPEditPos;
	unsigned int	LastNPMode;

	unsigned int	LastWLGroup;
	unsigned int	LastWLEntry;
	unsigned int	LastWLEditPos;
	unsigned int	LastWLMode;

	unsigned int	AgendaPtr;
	unsigned int	AgendaSize;

	unsigned int	AutofieldPtr;
	unsigned int	AutofieldSize;

	unsigned int	AddressbookPtr;
	unsigned int	AddressbookSize;

	unsigned int	NotesPtr;
	unsigned int	NotesSize;				

	unsigned int	INetLinksPtr;
	unsigned int	INetLinksSize;

	unsigned int	AgendaData;			// DATA_PTR must point to this offset
	unsigned int	AutofieldData;
	unsigned int	AddressbookData;
	unsigned int	NotesData;
	unsigned int	INetLinksData;

	char			pre0;				// File end
	char			Type[4];			// initialised with 'data'
	char			post0;
	char			Tag;				// OTH_TAG
} TDataFile;


typedef struct
{
	char			*DataPtr;
	TDataFile		*Header;
	HANDLE			handle;
	char			PathBuf[19];
	char			*Path;
} TFile;


typedef struct
{
	unsigned int	hashTbl[366 + 2*HASH_TBL_OVERLAP];
	int				year;
	unsigned int	firstDay;
} THashSet;


typedef struct
{
	int				NextTask;
	int				NextMode;
	unsigned int	prm0;					// day or groupnr (0 = first)
	unsigned int	prm1;					// datasetnr (-1 = none, 0 = first)
	int				keepTask;	
} TNextTask;


typedef struct
{
	TFile			DataFile;				// DataFile
	int				isTwin;					// true, if DataFile is archived -> make Twin
	char			BFPathBuffer[19];
	char			*BackupFilePath;
	int				Changed;				// true, if any changes were made. (EditDay and DeleteDay)
	TOnOff			MainCursor;
	int				Exit;					//
	int				OffOnExit;
	unsigned int	nStarts;				// gets in PrepareAutoBackup the last num of changes since last backup
	int				UseClock;				// use new clockfeatures (AMS >= 2.07; Clock enabled)
	int				OldInitKeyDelay;
	int				OldBetweenKeyDelay;
	TNextTask		TaskQueue;
	THashSet		hashSet;
	volatile int	TurnedOff;
	volatile int	keyCombo;				// used to check combinations with on-key
	INT_HANDLER 	OldInt6;				// used for ON-Int vector
} TAgenda;


register TAgenda 	*gr_Agenda asm ("a3");
extern TAgenda		*g_Agenda;

// --------------------------------------------------------------------------
// --- Prototypes -----------------------------------------------------------
// --------------------------------------------------------------------------

int			GetPassword ();
void		StartAgenda ();
void		initAgenda ();

// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

#endif

		 
		                 
