// C Source File

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Title:        AGENDA V4.2  oooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Author:       Adrian Bader o - o A.Bader@gmx.ch ooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
// -- Best viewed with TAB-STOP 4! ------------------------------------------
// --------------------------------------------------------------------------


#include "IncludeAll.h"

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int IsMainEventKey (int key)
{
	int		Ret = 0;

	if ( (key == KEY_MODE) ||
	     (key == KEY_F6) ||
	     (key == KEY_F7) ||
	     (key == KEY_F8)
	) Ret = 1;
	
	return Ret;
}

// --------------------------------------------------------------------------

unsigned int getTaskFromEntry (TEntryHeader *header)
{
	switch (header->Flags & FLAG_ET_TASK_MASK)
	{
		case FLAG_ET_AGENDA: return TASK_AGENDA;
		case FLAG_ET_ADDRESSBOOK: return TASK_ADDRESSBOOK;
		case FLAG_ET_NOTES: return TASK_NOTEPAD;
		case FLAG_ET_LINKS: return TASK_INET_LINKS;
	}

	return 0;
}

void handleShortcut (int key, int currentTask)
{
	int				Ret		= ER_SUCCESS;
	TEntryHeader	*header;
	int				task;
	int				isGroup;
	int				groupNr;
	int				datasetNr;
	TEntryType		ET;
	
	header = getAssoziatedShortcut (key);
	if (header)
	{
		isGroup = ((header->Flags & FLAG_ET_MASK) == FLAG_ET_GROUP);
		task = getTaskFromEntry (header);

		gr_Agenda->TaskQueue.keepTask = (currentTask == task);
		gr_Agenda->TaskQueue.NextTask = task;
		if (!isGroup) gr_Agenda->TaskQueue.NextMode = EDITOR_VIEW;

		if (task == TASK_AGENDA)
		{
			gr_Agenda->TaskQueue.prm0 = ((TAgendaHeader *)header)->Day;
		}
		else
		{
			InitEntryType (&ET, task);
			Ret = findEntry (&ET, header, &groupNr, &datasetNr);
			gr_Agenda->TaskQueue.prm0 = groupNr;
			gr_Agenda->TaskQueue.prm1 = datasetNr;
		}				

		gr_Agenda->Exit = TASK_CHANGE;
	}
	
	ShowErrors (Ret);
}

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void resetTaskQueueParams ()
{
	gr_Agenda->TaskQueue.keepTask = 0;
	gr_Agenda->TaskQueue.prm0 = TASK_NO_MESSAGE;
	gr_Agenda->TaskQueue.prm1 = TASK_NO_MESSAGE;
	gr_Agenda->TaskQueue.NextMode = 0;
}

void switchTaskTo (int task)
{
	resetTaskQueueParams();
	gr_Agenda->TaskQueue.NextTask = task;
	gr_Agenda->Exit = TASK_CHANGE;
}

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

DEFINE_INT_HANDLER(IntHandler6)
{
	int			down			= 0;
	int			x, y;
	TAgenda		*oldA3			= gr_Agenda;
	static int	keytable[8][10] =
		{
			{SND_KEY, 0, 0, KEY_STO, ' ', '/', '^', '=', KEY_BACKSPACE, '-'},
	 		{DIAMOND_KEY, 'z', 'x', 'c', 'v', 'b', 'n', 'm', THETA_KEY, KEY_ENTER},
	 		{SHIFT_KEY, 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'a'},
	 		{HAND_KEY, 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'q'},
	 		{LEFT_KEY, KEY_F8, KEY_F3, KEY_F7, KEY_F2, KEY_F6, KEY_F1, KEY_F5, '+', KEY_F4},
	 		{UP_KEY, '1', '4', '7', '(', SIN_KEY, LN_KEY, KEY_CLEAR, KEY_MODE, '0'},
	 		{RIGHT_KEY, '2', '5','8' ,')' ,COS_KEY, KEY_ENTER, KEY_APPS, KEY_ESC, '.'},
	 		{DOWN_KEY, '3', '6', '9', ',', TAN_KEY, 'p', '*' ,0, KEY_SIGN}
		};
	
	// --- if Diamond flag is set: shut down the calc
	if (ST_flags & 2)
	{
		ST_flags &= ~2;
		ST_showHelp ("");
		ST_eraseHelp();
		off();
	}
	
	// --- restore a3, the global Agenda struct pointer
	oldA3 = gr_Agenda;
	gr_Agenda = g_Agenda;
	// --- DON'T RESTORE A5 since no rom call is used yet
	
	while( (~(*(volatile unsigned char *)0x60001A) & 2) && !down)
	{
		for (x = 0; x < 8; x++)
			for (y = 0; y < 10; y++)
				if (_rowread(~((short)(1<<y))) & (1<<x)) down = keytable[x][y];
	}

	// --- acknowledge the interrupt
	*(volatile unsigned char *)0x60001A = 0;

	ExecuteHandler (gr_Agenda->OldInt6);
	gr_Agenda->keyCombo = down;

	gr_Agenda = oldA3;
}

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void IdleLoop (TIdleLoop *IL, int Flags, int *key, int *DidWhat, EVENT *ev)
{
	int		k;
	int		Tag1		= 0;
	int		Tag2		= 0;
	int		lDidWhat	= 0;
	int		Feedback;
	int		useEVgetc	= ev != NULL;		
	
	// --- delete the global flag for combination-detection with ON-Key
	gr_Agenda->keyCombo = 0;
	
	// --- install ON-Key combo detection
	SetIntVec (AUTO_INT_6, IntHandler6);

	if (!useEVgetc) k = GKeyIn (NULL, 0);
	else k = EV_getc (ACTIVITY_IDLE, ev);

	// --- redirect AutoInt6 back to its AMS handler
	SetIntVec (AUTO_INT_6, gr_Agenda->OldInt6);

	// --- handle a power off/on by initialising the keyboard speed
 	if (gr_Agenda->TurnedOff)
	{
		gr_Agenda->TurnedOff = 0;
		OSInitKeyInitDelay (KEY_INIT_DELAY);
		OSInitBetweenKeyDelay (BETWEEN_KEY_DELAY);
	}
	
	// --- reset values
	if (key != NULL) *key = 0;
	if (DidWhat != NULL) *DidWhat = 0;

	// --- handle key combinations with ON-Key
	if (gr_Agenda->keyCombo)
	{
		handleShortcut (gr_Agenda->keyCombo, IL->Task);
	}
	else
	{	
		if (key != NULL) *key = k;
	
		if (!(Flags & ILF_NOMENU) && IsMainEventKey (k))
		{
			if (IL->BeforeMenuHook) IL->BeforeMenuHook (IL->Ptr1, IL->Ptr2, &Tag1, &Tag2);
			Feedback = MainMenu(k, &lDidWhat);
			if (IL->AfterMenuHook) IL->AfterMenuHook (IL->Ptr1, IL->Ptr2, Tag1, Tag2, lDidWhat, Feedback);
		}
	
		if ( !(Flags & ILF_NO_OFF_ON_EXIT) && (k == KEY_2ND + KEY_ON) ) gr_Agenda->OffOnExit = gr_Agenda->Exit = IL->Task;
		if ( !(Flags & ILF_NO_EXIT) && (k == KEY_DIAMOND + KEY_ESC) ) gr_Agenda->Exit = IL->Task;
		if ( !(Flags & ILF_NO_OFF) && (k == KEY_DIAMOND + KEY_ON) ) off();
		
		if (!(Flags & ILF_NO_TASK_SWITCH))
		{
			// optimize ??
			if ( (k == KEY_F1) && (IL->Task != TASK_AGENDA)) switchTaskTo (TASK_AGENDA);
			if ( (k == KEY_F2) && (IL->Task != TASK_ADDRESSBOOK)) switchTaskTo (TASK_ADDRESSBOOK);
			if ( (k == KEY_F3) && (IL->Task != TASK_NOTEPAD)) switchTaskTo (TASK_NOTEPAD);
			if ( (k == KEY_F4) && (IL->Task != TASK_INET_LINKS)) switchTaskTo (TASK_INET_LINKS);
		}
	}
	
	if (DidWhat != NULL) *DidWhat = lDidWhat;

}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
