// C Source File

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Title:        AGENDA V4.2  oooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Author:       Adrian Bader o - o A.Bader@gmx.ch ooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
// -- Best viewed with TAB-STOP 4! ------------------------------------------
// --------------------------------------------------------------------------


#include "IncludeAll.h"

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void InitEntryType (TEntryType *ET, int Task)
{
	
	switch (Task)
	{
		case TASK_ADDRESSBOOK :
				ET->Task = Task;
				ET->EntryFlag = FLAG_ET_ADDRESSBOOK;
				ET->TaskTitle = AB_TITLE;
				ET->DatasetNames = NAME_LIST;
				ET->NewEntryName = LV_GM_ADDADDRESS;
				ET->DatasetMenuName = LV_DS_ADDRESS;
				ET->MsgDelDataset = MSG_DEL_ADDRESS;
				ET->NewEntryKeyCombo = KEY_DIAMOND + 'A';
				ET->NumStrings = 9;
				ET->GetBasePtr = &ABookGetBasePtr;
				ET->GetBaseSize = &ABookGetBaseSize;
				ET->AdjustSize = &AdjustABookSize;
				ET->CreateDataset = &CreateAddress;
				ET->GetDatasetList = &GetAddressNames;
				break;

		case TASK_NOTEPAD :
				ET->Task = Task;
				ET->EntryFlag = FLAG_ET_NOTES;
				ET->TaskTitle = NP_TITLE;
				ET->DatasetNames = NOTE_LIST;
				ET->NewEntryName = LV_GM_ADDNOTE;
				ET->DatasetMenuName = LV_DS_NOTE;
				ET->MsgDelDataset = MSG_DEL_NOTE;
				ET->NewEntryKeyCombo = KEY_DIAMOND + 'A';
				ET->NumStrings = 2;
				ET->GetBasePtr = &NotepadGetBasePtr;
				ET->GetBaseSize = &NotepadGetBaseSize;
				ET->AdjustSize = &AdjustNotepadSize;
				ET->CreateDataset = &CreateNote;
				ET->GetDatasetList = &GetNoteNames;
				break;

		case TASK_INET_LINKS :
				ET->Task = Task;
				ET->EntryFlag = FLAG_ET_LINKS;
				ET->TaskTitle = IL_TITLE;
				ET->DatasetNames = LINKS_LIST;
				ET->NewEntryName = LV_GM_ADDLINK;
				ET->DatasetMenuName = LV_DS_LINK;
				ET->MsgDelDataset = MSG_DEL_LINK;
				ET->NewEntryKeyCombo = KEY_DIAMOND + 'A';
				ET->NumStrings = 2;
				ET->GetBasePtr = &LinksGetBasePtr;
				ET->GetBaseSize = &LinksGetBaseSize;
				ET->AdjustSize = &AdjustLinksSize;
				ET->CreateDataset = &CreateNote;
				ET->GetDatasetList = &GetNoteNames;
				break;
			
	}	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void ReleasehList (TEntryType *ET, HANDLE *hBuf)
{
	int		n;
	
	for (n = 0; n < (ET->NumStrings); n++)
	{
		HeapFreeHNULL (&hBuf[n]);
	}
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

unsigned int GetSizeOfhList (TEntryType *ET, HANDLE *hBuf)
{
	unsigned int	size	= 0;
	int				i;
	
	for (i = 0; i < (ET->NumStrings); i++)
	{
		size += strlen (HeapDeref (hBuf[i])) +1;
	}
	size = RoundUpEven (size);

	return size;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int GetGroupCount (TEntryType *ET)
{
	int				GroupCounter	= 0;
	TEntryHeader	*Header;

	Header = (TEntryHeader *)ET->GetBasePtr ();
	while (Header->Flags != FLAG_ET_DATAEND)
	{
		if (Header->Flags & FLAG_ET_GROUP) GroupCounter++;
		Header = (TEntryHeader *)( (char *)Header + Header->Size);
	}

	return GroupCounter;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int GetGroups (TEntryType *ET, HANDLE *handle, int GroupNr, unsigned int maxLen)
{
// if GroupNr != -1, this Group, if valid, won't be included in the list
// if maxLen != 0 -> the strings will be truncated after maxLen if larger by
// inserting a null char.
	int				Ret				= ER_SUCCESS;
	int				offset			= 0;
	int				size 			= 0;
	int				GroupCounter	= 0;
	char			*ptr;
	TEntryHeader	*Header;
	
	*handle = HeapAlloc (2);
	if (*handle == H_NULL) Ret = ER_MALLOC;
	
	if (Ret == ER_SUCCESS)
	{	
		Header = (TEntryHeader *)ET->GetBasePtr ();
		while ( (Header->Flags != FLAG_ET_DATAEND) && (Ret == ER_SUCCESS) )
		{
			if (Header->Flags & FLAG_ET_GROUP)
			{
				if (GroupNr != GroupCounter)
				{
					if ( HeapRealloc (*handle, size += (Header->Size + 2)) == H_NULL) Ret = ER_MALLOC;
					if (Ret == ER_SUCCESS)
					{			
						ptr = HeapDeref (*handle) + offset;
						strcpy ( ptr, &Header->data);
						if (maxLen && strlen (&Header->data) > maxLen) poke (ptr + maxLen, 0); 
						offset += strlen (ptr) + 1;
					}
				}
				GroupCounter++;
			}
			Header = (TEntryHeader *)( (char *)Header + Header->Size);
		}
		poke ( HeapDeref (*handle) + offset++, 0);
		poke ( HeapDeref (*handle) + offset, 0);
		if (Ret != ER_SUCCESS)
		{
			HeapFreeHNULL (handle);
		}
	}
	
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int GetGroup (TEntryType *ET, char *name, int num, TEntryHeader **HeaderPtr)
{
	// --- name is optional
	int				Ret		= ER_SUCCESS;
	int				found	= 0;
	int				i		= 0;
	TEntryHeader	*Header;
	
	Header = (TEntryHeader *)ET->GetBasePtr ();
	while ( (Header->Flags != FLAG_ET_DATAEND) && !found)
	{
		if (Header->Flags & FLAG_ET_GROUP)
		{
			if (name != NULL)
			{
				if (strcmp (&Header->data, name) == 0)
				{
					found = 1;
					if (HeaderPtr != NULL) *HeaderPtr = Header;
				}
			}
			else
			{
				if (num == i)
				{
					found = 1;
					if (HeaderPtr != NULL) *HeaderPtr = Header;
				}
				i++;
			}
		}
		Header = (TEntryHeader *)((char *)Header + Header->Size);
	}
	if (!found) Ret = ER_GROUP_NOT_FOUND;
	
	return Ret;
}


// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int AddGroup (TEntryType *ET, char *name)
{
	int				Ret		= ER_SUCCESS;
	long			dSize;
	char			*EntryPos;
	TEntryHeader	*Header;
	
	Ret = GetGroup (ET, name, -1, NULL);
	if (Ret == ER_GROUP_NOT_FOUND) Ret = ER_SUCCESS;
	else Ret = ER_GROUP_EXISTS;

	if (Ret == ER_SUCCESS)
	{
		dSize = RoundUpEven (sizeof (TEntryHeader)-2 + strlen (name)+1);
		EntryPos = ET->GetBasePtr () + ET->GetBaseSize () -2;
		Ret = ReallocDataFile (EntryPos, gr_Agenda->DataFile.Header->Size +2 +dSize, &EntryPos);
		if (Ret == ER_SUCCESS)
		{
			Header = (TEntryHeader *)EntryPos;
			Header->Flags = ET->EntryFlag | FLAG_ET_GROUP; 
			Header->Shortcut = 0;
			Header->Size = (unsigned int)dSize;
			strcpy (&Header->data, name);			
			ET->AdjustSize (dSize);
		}
	}	

	return Ret;	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void TraceDSListToEnd (TEntryHeader **HeaderPtr)
{
	TEntryHeader *Header;
	
	Header = *HeaderPtr;
	
	Header = (TEntryHeader *)((char *)Header + Header->Size);
	while ( (Header->Flags & FLAG_ET_MASK) == FLAG_ET_ENTRY)
	{
		Header = (TEntryHeader *)((char *)Header + Header->Size);
	}
	
	*HeaderPtr = Header;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int DelGroup (TEntryType *ET, int GroupNr)
{
	int				Ret		= ER_SUCCESS;
	long			dSize;
	char			*Pos;
	TEntryHeader	*Header;
	
	Ret = GetGroup (ET, NULL, GroupNr, &Header);
	
	if (Ret == ER_SUCCESS)
	{
		Pos = (char *)Header;
		TraceDSListToEnd (&Header);
		dSize = Pos - (char *)Header;
		Ret = ReallocDataFile (Pos, gr_Agenda->DataFile.Header->Size +2 +dSize, NULL);  
		if (Ret == ER_SUCCESS) ET->AdjustSize(dSize);
	}	

	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int RenameGroup (TEntryType *ET, int GroupNr, char *Name)
{
	int				Ret		= ER_SUCCESS;
	long			dSize;
	int				headerSize;
	TEntryHeader	*Header;
	char			*newPos;
	unsigned int	shortcut;
	
	Ret = GetGroup (ET, Name, -1, NULL);
	{
		if (Ret != ER_GROUP_NOT_FOUND) Ret = ER_GROUP_EXISTS;
		else Ret = ER_SUCCESS;
	}
	if (Ret == ER_SUCCESS) Ret = GetGroup (ET, NULL, GroupNr, &Header);
	if (Ret == ER_SUCCESS)
	{
		headerSize = Header->Size;
		shortcut = Header->Shortcut;
		dSize = (strlen (Name) +1 ) - (headerSize - (sizeof(TEntryHeader) - 2) );
		dSize = RoundUpEven (dSize);
		Ret = ReallocDataFile ((char *)Header, gr_Agenda->DataFile.Header->Size +2 +dSize, &newPos);
		Header = (TEntryHeader *)newPos;
		if (Ret == ER_SUCCESS)
		{
			Header->Size = headerSize + (unsigned int)dSize;
			Header->Shortcut = shortcut;
			Header->Flags = ET->EntryFlag | FLAG_ET_GROUP;
			strcpy (&Header->data, Name);
			ET->AdjustSize (dSize);
		}
	}
	
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int GetDatasetCount (TEntryType *ET, int GroupNr, int *Count)
{
// if GroupNr == -1, all addresses will be counted
	int				Ret			= ER_SUCCESS;
	int				Counter		= 0;
	int				end			= 0;
	TEntryHeader	*Header;
	
	if (GroupNr == -1)
	{
		Header = (TEntryHeader *)ET->GetBasePtr ();
	}
	else
	{
		Ret = GetGroup (ET, NULL, GroupNr, &Header);
		Header = (TEntryHeader *)((char *)Header + Header->Size);
	}
		
	if (Ret == ER_SUCCESS)
	{
		while ( (Header->Flags != FLAG_ET_DATAEND) && !end) 
		{
			if ( (Header->Flags & FLAG_ET_MASK) == FLAG_ET_ENTRY) Counter++;
			else
			{
				if (GroupNr != -1) end = 1;
			}
			Header = (TEntryHeader *)((char *)Header + Header->Size);
		}	
		
		*Count = Counter;
	}

	return Ret;	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int GetDatasetPtr (TEntryType *ET, int GroupNr, int DatasetNr, TEntryHeader **HeaderPtr)
{
	int				Ret			= ER_SUCCESS;
	int				Counter		= 0;
	int				found		= 0;
	TEntryHeader	*Header;
	
	Ret = GetGroup (ET, NULL, GroupNr, &Header);
	
	if (Ret == ER_SUCCESS)
	{
		Header = (TEntryHeader *)((char *)Header + Header->Size);
		while ( ((Header->Flags & FLAG_ET_MASK) == FLAG_ET_ENTRY) && !found)
		{
			if (Counter == DatasetNr)
			{
				found = 1;
				*HeaderPtr = Header;
			}
			Header = (TEntryHeader *)((char *)Header + Header->Size);
			Counter++;
		}
		
		if (!found) Ret = ER_NAME_NOT_FOUND;
	}
		
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int AddDataset (TEntryType *ET, HANDLE *hBuf, int GroupNr, int DatasetNr, int OverWrite, TEntryHeader **newHdrpos, unsigned int shortcut)
{
	int				Ret			= ER_SUCCESS;
	TEntryHeader	*Header;
	long			dSize		= 0;
	int				same		= 0;
	int				i;
	char			*Pos;
	char			*data;
	char			*List[ET->NumStrings];
	char			*newPos;
	
	if (DatasetNr == -1)	// new entry. Trace to end
	{
		Ret = GetGroup (ET, NULL, GroupNr, &Header);
		if (Ret == ER_SUCCESS) TraceDSListToEnd (&Header);
	}
	else
	{						// overwrite existing entry if not same
		Ret = GetDataset (ET, GroupNr, DatasetNr, List, &Header);
		{					 // Test whether its same
			same = 1;
			for (i = 0; i < ET->NumStrings; i++)
			{
				if (strcmp (HeapDeref (hBuf[i]), List[i]) != 0) same = 0;
			}	
		}
		if ( !same && (Ret == ER_SUCCESS) && OverWrite )
		{
			dSize = (long)Header->Size * (-1);
			Ret = ReallocDataFile ((char *)Header, gr_Agenda->DataFile.Header->Size+2 + dSize, &newPos) ;
			Header = (TEntryHeader *)newPos;
			if (Ret == ER_SUCCESS) ET->AdjustSize (dSize); 
		}
	}

	if (!same && (Ret == ER_SUCCESS) )
	{
		dSize = GetSizeOfhList (ET, hBuf) + sizeof (TEntryHeader) - 2;
		Ret = ReallocDataFile ((char *)Header, gr_Agenda->DataFile.Header->Size+2 + dSize, &newPos) ;

		Header = (TEntryHeader *)newPos;
		// --- if newHdrPos passed, save new position
		if (newHdrpos != NULL) *newHdrpos = Header;
		
		if (Ret == ER_SUCCESS)
		{
			Header->Flags = ET->EntryFlag | FLAG_ET_ENTRY;
			Header->Shortcut = shortcut;
			Header->Size = (unsigned int)dSize;
			Pos = &Header->data;
			for (i = 0; i < ET->NumStrings; i++)
			{
				data = HeapDeref (hBuf[i]);
				strcpy (Pos, data);
				Pos += 	strlen (data) +1;
			}
			ET->AdjustSize (dSize);
		}
	}
			
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int DelDataset (TEntryType *ET, int GroupNr, int DatasetNr)
{
	int				Ret			= ER_SUCCESS;
	long			dSize;
	char			*Pos;
	TEntryHeader	*Header;

	Ret = GetDatasetPtr (ET, GroupNr, DatasetNr, &Header);
	
	if (Ret == ER_SUCCESS)
	{
		Pos = (char *)Header;
		dSize = (long)Header->Size * (-1);				

		Ret = ReallocDataFile (Pos, gr_Agenda->DataFile.Header->Size +2 +dSize, NULL);  
		if (Ret == ER_SUCCESS) ET->AdjustSize (dSize);
	}

	return Ret;	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int MoveDataset (TEntryType *ET, int GroupNr, int DatasetNr, int DestGroup, int DestDataset)
{
	int				Ret						= ER_SUCCESS;
	unsigned int	shortcut				= 0;
	HANDLE			hList[ET->NumStrings];
	int				Count;
	TEntryHeader	*header;
	

	if ( !((GroupNr == DestGroup) && (DatasetNr == DestDataset)) )
	{
		Ret = GetDatasetInHandles (ET, GroupNr, DatasetNr, hList, &header);
		
		if (Ret == ER_SUCCESS)
		{
			shortcut = header->Shortcut;
			Ret = DelDataset (ET, GroupNr, DatasetNr);
		}
				
		if (Ret == ER_SUCCESS)
		{
			Ret = GetDatasetCount (ET, DestGroup, &Count); 
			if (DestDataset == Count) DestDataset = -1;					// would point behind last entry. would raise error 0x23
		}																// -1 means, that it becomes last entry anyway
		if (Ret == ER_SUCCESS) Ret = AddDataset (ET, hList, DestGroup, DestDataset, 0, NULL, shortcut);
	
		ReleasehList (ET, hList);
	}

	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int GetDataset (TEntryType *ET, int GroupNr, int DatasetNr, char **List, TEntryHeader **EntryHeader)
{
// EntryHeader is optional
	int				Ret			= ER_SUCCESS;
	int				i;
	char			*pos;
	TEntryHeader	*Header;
	
	Ret = GetDatasetPtr (ET, GroupNr, DatasetNr, &Header);
	if (Ret == ER_SUCCESS)
	{
		if (EntryHeader) *EntryHeader = Header;
		pos = &Header->data;
		for (i = 0; i < ET->NumStrings; i++)
		{
			List[i] = pos;
			pos += strlen (pos) +1;
		}
	}

	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int GetDatasetInHandles (TEntryType *ET, int GroupNr, int DatasetNr, HANDLE *hBuf, TEntryHeader **EntryHeader)
{
// --- EntryHeader is optional. May be NULL.
	int			Ret			= ER_SUCCESS;
	char		*List[ET->NumStrings];
	int			n;
	
	Ret = GetDataset (ET, GroupNr, DatasetNr, List, EntryHeader);
	if (Ret == ER_SUCCESS)
	{
		for (n = 0; n < ET->NumStrings; n++)
		{
			if ( (hBuf[n] = HeapAlloc (strlen (List[n])+1)) == H_NULL ) Ret = ER_MALLOC;
			else strcpy (HeapDeref (hBuf[n]), List[n]);
		}
	}

	if (Ret == ER_MALLOC) ReleasehList (ET, hBuf);
	
	return Ret;
}

// --------------------------------------------------------------------------

int findEntry (TEntryType *ET, TEntryHeader *entry, int *GroupNr, int *DatasetNr)
{
	int				Ret		= ER_SUCCESS;
	int				found	= 0;
	TEntryHeader	*header;
	
	*GroupNr = -1;
	header = (TEntryHeader *)ET->GetBasePtr ();
	while ( (header->Flags != FLAG_ET_DATAEND) && !found)
	{
		if ( (header->Flags & FLAG_ET_MASK) == FLAG_ET_GROUP)
		{
			(*GroupNr)++;
			*DatasetNr = -1;
		}
		else (*DatasetNr)++;

		if (header == entry) found = 1;

		header = (TEntryHeader *)((char *)header + header->Size);
	}
	
	if (!found) Ret = ER_ENTRY_NOT_FOUND;
	return Ret;		
}


// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
