// C Header File

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Title:        AGENDA V4.2  oooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Author:       Adrian Bader o - o A.Bader@gmx.ch ooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
// -- Best viewed with TAB-STOP 4! ------------------------------------------
// --------------------------------------------------------------------------


#ifndef		__HDFENTRY__
#define		__HDFENTRY__

// --------------------------------------------------------------------------
// --- Structures -----------------------------------------------------------
// --------------------------------------------------------------------------

typedef struct TEntryType
{
	int				Task;
	unsigned int	EntryFlag;
	char			*TaskTitle;
	char			*DatasetNames;
	char			*NewEntryName;
	char			*DatasetMenuName;
	char			*MsgDelDataset;
	int				NewEntryKeyCombo;
	int				NumStrings;
	char			*(*GetBasePtr)();
	unsigned int	(*GetBaseSize)();
	void			(*AdjustSize)(long int SizeDelta);	
	int				(*GetDatasetList)(struct TEntryType *ET, int GroupNr, HANDLE *handle);
	int 			(*CreateDataset)(struct TEntryType *ET, HANDLE *hBuf, int GroupNr);
} TEntryType;



typedef struct
{
	unsigned int	Flags;
	unsigned int	Shortcut;
	unsigned int	Size;
	char			data;
} TEntryHeader;


// --------------------------------------------------------------------------
// --- Prototypes -----------------------------------------------------------
// --------------------------------------------------------------------------

void			InitEntryType (TEntryType *ET, int Task);

void			ReleasehList (TEntryType *ET, HANDLE *hBuf);
unsigned int 	GetSizeOfhList (TEntryType *ET, HANDLE *hBuf);

int 			GetGroupCount (TEntryType *ET);
int 			GetGroups (TEntryType *ET, HANDLE *handle, int GroupNr, unsigned int maxLen);
int 			GetGroup (TEntryType *ET, char *name, int num, TEntryHeader **HeaderPtr);
int 			AddGroup (TEntryType *ET, char *name);
void			TraceDSListToEnd (TEntryHeader **HeaderPtr);
int				DelGroup (TEntryType *ET, int GroupNr);
int 			RenameGroup (TEntryType *ET, int GroupNr, char *Name);

int				GetDatasetCount (TEntryType *ET, int GroupNr, int *Count);
int 			GetDatasetPtr (TEntryType *ET, int GroupNr, int DatasetNr, TEntryHeader **HeaderPtr);
int 			AddDataset (TEntryType *ET, HANDLE *hBuf, int GroupNr, int DatasetNr, int OverWrite, TEntryHeader **newHdrpos, unsigned int shortcut);
int 			MoveDataset (TEntryType *ET, int GroupNr, int DatasetNr, int DestGroup, int DestDataset);
int 			DelDataset (TEntryType *ET, int GroupNr, int DatasetNr);
int 			GetDataset (TEntryType *ET, int GroupNr, int DatasetNr, char **List, TEntryHeader **EntryHeader);
int				GetDatasetInHandles (TEntryType *ET, int GroupNr, int DatasetNr, HANDLE *hBuf, TEntryHeader **EntryHeader);
int 			findEntry (TEntryType *ET, TEntryHeader *entry, int *GroupNr, int *DatasetNr);

// --- data end isn't combined with the tast identifier because else there wouldn't
// --- be a 0000 after each tasks data
#define		FLAG_ET_DATAEND			0

// --- the two flags are exclusive. Only one can apply for each entry's Flag.
// --- The two most LSBs are used for this.
#define		FLAG_ET_GROUP			1
#define		FLAG_ET_ENTRY			2
#define		FLAG_ET_MASK			3

// --- to each entry's Flag, one of this must be ored to.
#define		FLAG_ET_AGENDA			4
#define		FLAG_ET_ADDRESSBOOK		8
#define		FLAG_ET_NOTES			16
#define		FLAG_ET_LINKS			32
#define		FLAG_ET_TASK_MASK		0b00011100

// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo

#endif

