// C Source File

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Title:        AGENDA V4.2  oooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oo Author:       Adrian Bader o - o A.Bader@gmx.ch ooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
// -- Best viewed with TAB-STOP 4! ------------------------------------------
// --------------------------------------------------------------------------


#include "IncludeAll.h"

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int	InitHandles (HANDLE hBuf[], int num)
{
	int		Ret = ER_SUCCESS;
	int		i;
	
	for (i = 0; i < num; i++)
	{
		if (hBuf[i] == H_NULL)
		{
			hBuf[i] = HeapAlloc (1);
			if (hBuf[i] != H_NULL) poke (HeapDeref (hBuf[i]), 0);
			else Ret = ER_MALLOC;
		}
	}
	
	return Ret;
}

char *strncpyex (char *dest, char *source, unsigned long n)
{
	unsigned long	i;
	
	for (i = 0; (i < n) && (*source != 0); i++)
	{
		*(dest++) = *(source++);
	}
	*dest = 0;
	
	return dest;
}


void truncTrail (char *string)
{
	while (*string != 0 && (*string > '9' || *string < '0'))
	{
		*string = '0';
		string++;
	}
}

// --------------------------------------------------------------------------

char getNumDelimiter (char *string)
{
	unsigned int	i;
	char			ch	= 0;
	
	for (i = 0; i < strlen (string) && !ch; i++)
	{
		if (string[i] > '9' || string[i] < '0') ch = string[i];
	}
	
	return (ch);
}

// --------------------------------------------------------------------------

void fillDisplay ()
{
	int		i;

	randomize();
	for (i = 0; i < LCD_SIZE - LCD_WIDTH; i++)	// Don't mess up status line
	{
		*((char *)LCD_MEM + i) = random (256);
	}	
}


void WinCloseEx (WINDOW *win)
{
	if (win->Flags & WF_SAVE_SCR) BitmapGet (&win->Window, HeapDeref (win->DupScr));
	WinClose (win);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Reallocates the Handle  ----------------------------------------------
// --- if NewSize is larger than the old, the handle will be enlarged, else it
// --- will be smallered.
// --- Pos points to the place, where the data will be inserted, or where
// --- data should be deleted. NewPos ^= Pos after Deref.
// --------------------------------------------------------------------------

int ReallocHandle (HANDLE handle, unsigned int oldSize, unsigned long newSize, char *pos, char **newPos)
{
	int				Ret			= ER_SUCCESS;
	unsigned int	nToMove;
	long int		dPos;
	char			*oldPos		= HeapDeref (handle);
	
	if (newSize > oldSize)
	{
		nToMove = oldSize - (pos - oldPos);
		if (HeapRealloc (handle, newSize) == H_NULL) Ret = ER_MALLOC;
		else
		{
			dPos = (char *)HeapDeref (handle) - oldPos;
			pos += dPos;
			memmove (pos + newSize - oldSize, pos, nToMove);
		}			
	}
	else
	{
		nToMove = newSize - (pos - oldPos);
		memmove (pos, pos + oldSize - newSize, nToMove);
		if (HeapRealloc (handle, newSize) == H_NULL) Ret = ER_MALLOC;
		dPos = (char *)HeapDeref (handle) - oldPos;
		pos += dPos;
	}

	if (newPos != NULL) *newPos = pos;

	return (Ret);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void HeapFreeHNULL (HANDLE *handle)
{
	if (*handle != H_NULL)
	{
		HeapFree (*handle);
		*handle = H_NULL;
	}
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

long int RoundUpEven (long int Num)
{
	if ( ((float)Num / 2.0) != ( Num / 2) ) Num++;
	
	return Num;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void printfn (const char *data, int n)
{
	char	Buf[200];
	
	memcpy (Buf, data, n);
	Buf[n] = 0;
	printf (Buf);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

char *strfill (char *dest, char *source, unsigned long n, char ch)
{
	unsigned long	len;
	
	memmove (dest, source, n);
	dest[n] = '\0';
	if ( (len = strlen (dest)) < n )
	{
		memset (dest + len, ch, n - len );
	}
	
	return dest;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

unsigned int strlenWithoutSpace (char *data)
{
	unsigned int	num = 0;
	
	while (*data)
	{
		if (*data != ' ') num++;
		*data++;
	}
	
	return num;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int SaveScreen (HANDLE *handle)
{
	int		Ret = ER_SUCCESS;
	
	*handle = HeapAlloc (LCD_SIZE);
	if (*handle == H_NULL) Ret = ER_MALLOC;
	else LCD_save (HeapDeref (*handle));
	
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void RestoreScreen (HANDLE *handle)
{
	if (*handle != H_NULL)
	{
		LCD_restore (HeapDeref (*handle));
		HeapFreeHNULL (handle);
	}
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void ShowFatalError (int Error)
{
	short		Font = FontSetSys (F_8x10);

	clrscr();
	printf ("\n\n Fatal Error.\n Perhaps there is not enough\n free memory.\n\n");
	printf (" Errornumber is %d\n",Error);
	ngetchx();

	FontSetSys (Font);	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

void ShowErrors (int Error)
{
	char	CommonError[50];
	
	Error &= 0xFF;
	memset (CommonError, 0, sizeof(CommonError));
	sprintf (CommonError, "%s : Nr. %d", MSG_ER_COMMON, Error);

	switch (Error)
	{
		case	ER_SUCCESS :
				break;

		case	ER_MALLOC :
				MsgBox (MSG_ER_TITLE, MSG_ER_MEM, BT_OK, BT_NONE);
				break;

		case	ER_CREATE_DATAFILE :
				MsgBox (MSG_ER_TITLE, MSG_ER_CREATE_DF, BT_OK, BT_NONE);
				break;
				
		case	ER_WRONG_PW :
				MsgBox (MSG_ER_TITLE, MSG_ER_WRONG_PW, BT_OK, BT_NONE);
				break;
				
		case	ER_NO_BACKUPFILE :
				MsgBox (MSG_ER_TITLE, MSG_ER_RST_BACKUP, BT_OK, BT_NONE);
				break;		
				
		case	ER_WRONG_DF_VERSION :
				MsgBox (MSG_ER_TITLE, MSG_ER_DATAFILE, BT_OK, BT_NONE);
				break;		
		
		default : MsgBox (MSG_ER_TITLE, CommonError, BT_OK, BT_NONE);
				
	}
	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Encodes the Data with Key -------------------------------------------
// --------------------------------------------------------------------------

void EncodeData (char *Data, long int Key, unsigned int Num)
{
	unsigned int	n = 0;
	TEncodeKey		EncodeKey;
	
	EncodeKey.Key = Key;
	
	while (n < Num)
	{
		*Data ^= EncodeKey.Byte[n % 4];
		n++;
		Data++;
	}
	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Splits a String into zeroterminated single-Strings. ------------------
// --- handle must be freed in caller function ------------------------------
// --- Returns num strings in stringlist in buffer --------------------------
// --- if String is empty, return 0 -----------------------------------------
// --- if last substring is empty, don't count it ---------------------------
// --------------------------------------------------------------------------

int SplitString (char *String, char Split, HANDLE *handle)
{
	int		nRows	= 0;
	char	*dest;
	
	*handle = HeapAlloc (strlen (String)+1 );
	if (*handle)
	{
		dest = HeapDeref (*handle);
		if (*String != 0) nRows++;
		while (*String != 0)
		{
			*dest = *String;
			String++;
			if (*dest == Split)
			{
				*dest = 0;
				if (*String != 0) nRows++;
			}		
			dest++;
		}
		*dest = 0;
	}
	
	return nRows;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Returns a Pointer used for SYM_NAME ----------------------------------
// --- Buffer : must be in minimum 19 Bytes! --------------------------------
// --- Folder : Foldername of File ------------------------------------------
// --- File   : Filename ----------------------------------------------------
// --------------------------------------------------------------------------

char *GetSymnamePtr (char *Buffer, char *Folder, char *File)
{
	int				Size;
	char			Empty[] = "";
	
	if (Folder == NULL) Folder = Empty;
	memset (Buffer, 0, 19);
	strcpy (Buffer+1, Folder);
	strcat (Buffer+1, "\\"); 
	strcat (Buffer+1, File);
	Size = strlen (Buffer+1)+1;
	
	return (Buffer + Size);
}
	
// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int CopySym (SYM_STR Dest, SYM_STR Source, char **DestF)
// Warning: After CopySym, HSYMs may become invalid, because of SymAdd!!	
{
	int				Ret			= ER_SUCCESS;
	HSYM			hsymDest;
	HSYM			hsymSource;
	HANDLE			hSource		= H_NULL;
	HANDLE			hDest		= H_NULL;
	char			*pDest;
	unsigned long	Size		= 0;
	
	hsymSource = SymFind (Source);
	if (!hsymSource.folder) Ret = ER_COPY_SYM;
	if (Ret == ER_SUCCESS)
	{
		hSource = DerefSym(hsymSource)->handle;
	}
	hsymDest = SymAdd (Dest);
	if (!hsymDest.folder) Ret = ER_COPY_SYM;
	if (Ret == ER_SUCCESS)
	{
		Size = peek_w( HeapDeref(hSource))+2;
		hDest = HeapAlloc (Size);
		if (hDest == H_NULL) Ret = ER_MALLOC;
	}
	if (Ret == ER_SUCCESS)
	{
		DerefSym (hsymDest)->handle = hDest;
		pDest = HeapDeref(hDest);
		if (DestF != NULL) *DestF = pDest;
		memmove (pDest, HeapDeref(hSource), Size);
	}
	return (Ret);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int	GetSymAddr (SYM_STR Sym, char **Data)
{
	int			Ret		= ER_SUCCESS;
	HSym		HSymF;
	SYM_ENTRY	*se;
	HANDLE		handle;
	
	HSymF = SymFind (Sym);
	if (HSymF.folder)
	{
		se = DerefSym (HSymF);
		handle = se->handle;
		*Data = HeapDeref (handle);
	}
	else Ret = ER_SYM_NOT_FOUND;
	
	return Ret;	
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int	GetSymSize (SYM_STR Sym, unsigned int *Size)
{
	int		Ret;
	char	*Data	= NULL;
	
	Ret = GetSymAddr (Sym, &Data);
	if (Ret == ER_SUCCESS)
	{
		*Size = peek_w (Data)+2;
	}
	
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int DelArchivedSym (SYM_STR Sym)
{
	int		Ret;
	HSYM	HSymF;
	
	HSymF = SymFind (Sym);
	if (HSymF.folder)
	{
		Ret = EM_moveSymFromExtMem (Sym, HS_NULL);
		if (Ret) Ret = SymDel (Sym);
		if (!Ret) Ret = ER_UNARCHIVE_SYM;
		else Ret = ER_SUCCESS;
	}
	else Ret = ER_SYM_NOT_FOUND;
	
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

char *GetStrFromList (char *data, unsigned int num)
{
	unsigned int	i;
	char			*ptr;
	
	ptr = data;
	for (i = 0; (i < num) && (*ptr != 0); i++)
	{
		ptr += strlen (ptr)+1;
	}
	if (*ptr == 0) ptr = NULL;
		
	return ptr;
}

char *GetStrFromListEx (char *data, unsigned int num)
{
	unsigned int	i;
	char			*ptr;
	
	ptr = data;
	for (i = 0; (i < num); i++)
	{
		ptr += strlen (ptr)+1;
	}
		
	return ptr;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

unsigned int GetNumStrs (char *data)
{
	unsigned int i	= 0;
	
	while (GetStrFromList (data, i) != NULL)
		i++;
		
	return i;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------


void SL_DrawCursor (TScrollList *SL, int idx)
{
	if (SL->CursorIndex != -1)
	{
		SL->srCursor.xy.y0 = SL->yRefPos + idx * SL->LineWidth;
		SL->srCursor.xy.y1 = SL->srCursor.xy.y0 + SL->LineWidth-3;
		ScrRectFill (&SL->srCursor, ScrRect, A_XOR);
	}
}

// --------------------------------------------------------------------------

void SL_DrawLostFocus (TScrollList *SL, int idx)
{
	WIN_RECT	wr;
	
	if (SL->CursorIndex != -1)
	{
		wr.x0 = SL->srCursor.xy.x0-1;
		wr.x1 = SL->srCursor.xy.x1+1;
		wr.y0 = SL->yRefPos + idx * SL->LineWidth-1;
		wr.y1 = wr.y0 + SL->LineWidth-1;
		DrawClipRect (&wr, ScrRect, A_XOR);
	}
	
}

// --------------------------------------------------------------------------

void SL_Scroll (TScrollList *SL)
{
	LCD_BUFFER	ScrBuf;
	char		*row;
	char		rowbuf[100];
	SCR_RECT	*srGetBmp;
	SCR_RECT	*srPutBmp;
	
	SL_DrawCursor (SL, SL->CursorIndex);
	if (SL->ListIndex < SL->LIbefore)
	{
		srGetBmp = &SL->srScrollDown; 
		srPutBmp = &SL->srScrollUp;
	}
	else
	{
		srGetBmp = &SL->srScrollUp;
		srPutBmp = &SL->srScrollDown;
	}
		
	BitmapGet (srGetBmp, ScrBuf);
	BitmapPut (srPutBmp->xy.x0, srPutBmp->xy.y0, ScrBuf, ScrRect, A_REPLACE);
	
	row = GetStrFromList (SL->data, SL->CursorIndex + SL->ListIndex);
	strfill (rowbuf, row, SL->cols, ' ');
	DrawStrXY (SL->xPrintPos, SL->yRefPos + SL->CursorIndex * SL->LineWidth, rowbuf, A_REVERSE);
}

// --------------------------------------------------------------------------

void ScrollListClearLines (TScrollList *SL, int a, int b)
{
	char	buf[100];
	int		i;
	
	memset (buf, ' ', sizeof(buf));
	poke (buf + SL->cols, '\0');
	a--;
	for (i = a; (i < b) && (i < SL->rows); i++)
	{
		DrawStrXY (SL->xPrintPos, SL->yRefPos + i*SL->LineWidth, buf, A_REPLACE);
	}
}

// --------------------------------------------------------------------------

void ScrollListRedraw (TScrollList *SL, int useCursor)
{
	int 	i;
	char	*row;
	char	rowbuf[100];
	short	attr;
	
	if ( (SL->fgHintifEmpty) && (SL->CursorIndex == -1) )
	{
		strfill (rowbuf, EMPTY, SL->cols, ' ');
		DrawStrXY (SL->xPrintPos, SL->yRefPos, rowbuf, A_REPLACE);
		ScrollListClearLines (SL, 2, SL->rows); 
	}
	else
	{
		for (i = 0; i < SL->rows; i++)
		{
			row = GetStrFromList (SL->data, i + SL->ListIndex);
			strfill (rowbuf, row, SL->cols, ' ');
			if (i == SL->CursorIndex && (useCursor == CU_NORMAL) && !SL->fgDrawOnly) attr = A_REVERSE;
			else attr = A_REPLACE;
			DrawStrXY (SL->xPrintPos, SL->yRefPos + i*SL->LineWidth, rowbuf, attr);
		}
		
		SL->CIbefore = SL->CursorIndex;
		SL->LIbefore = SL->ListIndex;
		if (!SL->fgDrawOnly) SL->Redraw = 0;
		if (useCursor == CU_LOSTFOCUS)
		{
			SL_DrawLostFocus (SL, SL->CursorIndex);
		}
	}
	SL->Cursor = useCursor;
}

// --------------------------------------------------------------------------

void SL_EvalKey (TScrollList *SL)
{
	int		key = SL->key;
	
	if (key == KEY_DOWN) SL->CursorIndex++;
	if (key == KEY_UP) SL->CursorIndex--;
	if (key == KEY_2ND + KEY_DOWN)
	{
		SL->CursorIndex += SL->rows;
		SL->Redraw = 1;
	}
	if (key == KEY_2ND + KEY_UP)
	{
		SL->CursorIndex -= SL->rows;
		SL->Redraw = 1;
	}
	if (key == KEY_DIAMOND + KEY_DOWN)
	{
		SL->CursorIndex = SL->rows-1;
		SL->ListIndex = SL->NumLines - SL->CursorIndex-1;
		SL->Redraw = 1;
	}
	if (key == KEY_DIAMOND + KEY_UP)
	{
		SL->CursorIndex = 0;
		SL->ListIndex = 0;
		SL->Redraw = 1;
	}

	if (SL->CursorIndex < 0)
	{
		SL->ListIndex += SL->CursorIndex;
		SL->CursorIndex = 0;
	}
	if (SL->CursorIndex > (SL->rows-1) )
	{
		SL->ListIndex += (SL->CursorIndex - (SL->rows-1));
		SL->CursorIndex = SL->rows-1;
	}
	if (SL->ListIndex < 0) SL->ListIndex = 0;
	if (SL->ListIndex > (SL->NumLines - 1 - SL->CursorIndex) ) SL->ListIndex = SL->NumLines - 1 - SL->CursorIndex;
	if (SL->CursorIndex > (SL->NumLines-1) )
	{
		SL->CursorIndex = SL->NumLines-1;
		SL->ListIndex = 0;
	}
}

// --------------------------------------------------------------------------

void ScrollListReassignData (TScrollList *SL, char *data)
{
	SL->data = data;
	SL->NumLines = GetNumStrs (data);

	if (SL->CursorIndex == -1) SL->CursorIndex = 0;

	if (SL->NumLines <= SL->rows)
	{
		SL->ListIndex = 0;
		if ( SL->CursorIndex > (SL->NumLines - 1) ) SL->CursorIndex = SL->NumLines - 1;
	}
	else
	{
		if ( (SL->CursorIndex > (SL->rows-1)) ||
		     ((SL->CursorIndex + SL->ListIndex) > (SL->NumLines-1))
		   )
		{
			SL->CursorIndex = SL->rows -1;
			SL->ListIndex = SL->NumLines - SL->CursorIndex -1;
		}
		
		if (SL->ListIndex >= (SL->NumLines - SL->rows))
		{
			SL->ListIndex = SL->NumLines - SL->rows;
		}

	}	

}

// --------------------------------------------------------------------------

void ScrollListCursorOff (TScrollList *SL)
{
	if (SL->Cursor == CU_NORMAL) SL_DrawCursor (SL, SL->CursorIndex);
	if (SL->Cursor == CU_LOSTFOCUS) SL_DrawLostFocus (SL, SL->CursorIndex);

	SL->Cursor = CU_NONE;
}

// --------------------------------------------------------------------------

TScrollList InitScrollList (int Task, int x0, int y0, int cols, int rows, char *data, int cIdx, int lIdx, int Flags)
{
	TScrollList		SL;
	int				xOfset;
	int				yOfset;
	
	SL.x0 = x0;
	SL.y0 = y0;
	SL.cols = cols;
	SL.rows = rows;
	SL.ListIndex = lIdx;
	SL.CursorIndex = cIdx;
	SL.LineWidth = 2 * FontGetSys () + 6 + 2;
	SL.CharWidth = SL.LineWidth - 4;
	SL.key = 0;
	SL.Redraw = 1;
	SL.Cursor = CU_NONE;
	ScrollListReassignData (&SL, data);
			
	SL.fgBorder = Flags & SL_BORDER;
	SL.fgDontClear = Flags & SL_DONT_CLEAR;
	SL.fgCallback = Flags & SL_CALLBACK;
	SL.fgDrawOnly = Flags & SL_DRAW_ONLY;
	SL.fgHintifEmpty = Flags & SL_HINT_IF_EMPTY;

	xOfset = 3*SL.fgBorder;
	yOfset = 3*SL.fgBorder;
	SL.xPrintPos = x0 + xOfset;	
	SL.yRefPos = y0 + yOfset;

	SL.wrBorder.x0 = x0;
	SL.wrBorder.x1 = x0 + cols*SL.CharWidth + 2*xOfset;
	SL.wrBorder.y0 = y0;
	SL.wrBorder.y1 = y0 + rows*SL.LineWidth + 2*yOfset-2;
	
	SL.srRect.xy.x0 = x0;
	SL.srRect.xy.x1 = SL.wrBorder.x1;
	SL.srRect.xy.y0 = y0;
	SL.srRect.xy.y1 = SL.wrBorder.y1;
	
	SL.srCursor.xy.x0 = x0 + xOfset;
	SL.srCursor.xy.x1 = x0 + xOfset + cols * SL.CharWidth-1;

	SL.srScrollDown.xy.x0 = SL.srCursor.xy.x0; 
	SL.srScrollDown.xy.x1 = SL.srCursor.xy.x1;
	SL.srScrollDown.xy.y0 = SL.yRefPos;
	SL.srScrollDown.xy.y1 = SL.yRefPos + (rows-1)*SL.LineWidth-2;

	SL.srScrollUp.xy.x0 = SL.srCursor.xy.x0; 
	SL.srScrollUp.xy.x1 = SL.srCursor.xy.x1;
	SL.srScrollUp.xy.y0 = SL.yRefPos + SL.LineWidth;;
	SL.srScrollUp.xy.y1 = SL.yRefPos + rows*SL.LineWidth-2;

	SL.idl.BeforeMenuHook = NULL;
	SL.idl.AfterMenuHook = NULL;
	SL.idl.Task = Task;

	return SL;
}

// --------------------------------------------------------------------------

int ScrollListSetCursor (TScrollList *SL, int idx)
{
	if (idx >= SL->NumLines) idx = 0;
	
	if (idx >= SL->rows)
	{
		SL->CursorIndex = SL->rows - 1;	
		SL->ListIndex = idx - SL->rows + 1;
	}
	else
	{
		SL->ListIndex = 0;
		SL->CursorIndex = idx;
	}
	
	return idx;
}

void ScrollList (TScrollList *SL, int *Index, int idlFlags)
{
// --- idlFlags: Flags, which are overgiven to IdleLoop to control possible
// --- allowed events. E.g. ILF_NO_TASK_SWITCH

	int			key		= 0;
	int			Exit	= 0;
	
	if (SL->Cursor != CU_NORMAL)
	{
		ScrollListCursorOff (SL);
		SL_DrawCursor (SL, SL->CursorIndex);
		SL->Cursor = CU_NORMAL;
	}

	if (!SL->fgDontClear) ScrRectFill (&SL->srRect, ScrRect, A_REVERSE);
	if (SL->fgBorder) DrawClipRect (&SL->wrBorder, ScrRect, A_NORMAL);
	
	while (key != KEY_ESC && key != KEY_ENTER && !Exit)
	{
		if (SL->Redraw | SL->fgDrawOnly) ScrollListRedraw (SL, 1);
		if (SL->CursorIndex != SL->CIbefore)
		{
			SL_DrawCursor (SL, SL->CIbefore);
			SL_DrawCursor (SL, SL->CursorIndex);
		}
		if (SL->ListIndex != SL->LIbefore) SL_Scroll (SL);
				
		if (!SL->fgDrawOnly) IdleLoop (&SL->idl, idlFlags, &key, NULL, NULL);
		else
		{
			Exit = 1;
			SL->fgDrawOnly = 0;
		}

		SL->CIbefore = SL->CursorIndex;
		SL->LIbefore = SL->ListIndex;
		SL->key = key;

		SL_EvalKey (SL);

		if (SL->fgCallback)
		{
			Exit = 1;
		}
		
	}
	if (Index != NULL) *Index = SL->ListIndex + SL->CursorIndex;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Draws a Border with the cordinates (x0,y0) (x1,y1) -------------------
// --------------------------------------------------------------------------

void DrawBorder (short x0, short y0, short x1, short y1)
{
	WIN_RECT	rectt = {x0,y0,x1,y1};
	WIN_RECT	rectd = {x0+3, y0+3, x1-3, y1-3};
	WIN_RECT	recte = {x0+2,y0+2,x1-2,y1-2};
	DrawClipRect (&rectt, ScrRect, A_NORMAL | B_DOUBLE);
	DrawClipRect (&rectd, ScrRect, A_NORMAL);
	DrawClipRect (&recte, ScrRect, A_REVERSE);
}

// --------------------------------------------------------------------------

void DrawFilledRect (short x0, short y0, short x1, short y1)
{
	WIN_RECT	recta = {x0,y0,x1,y1};
	SCR_RECT	recti = {{x0+1, y0+1, x1-1, y1-1}};

	DrawClipRect (&recta, ScrRect, A_NORMAL);
	ScrRectFill (&recti, ScrRect, A_REVERSE);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --- Shows a Messagebox ---------------------------------------------------
// ---	Text is a string with \0 chars after each line to display. ----------
// --------------------------------------------------------------------------

int MsgBox (char *Title, char *Text, short LeftButton, short RightButton )
{
	HANDLE			DlgHandle;
	HANDLE			handle		= H_NULL;
	int				i;
	char			*ptr;
	unsigned int	maxlen		= 0;
	int				Ret			= ER_SUCCESS;
	int				Width;
	int				Rows		= 0;
	
	ptr = Text;
	while (*ptr != '\0')
	{
		if (strlen (ptr) > maxlen) maxlen = strlen (ptr);
		ptr += strlen (ptr)+1;
		Rows++;
	}
	if (Rows > MSG_BOX_MAX_ROWS) Rows = MSG_BOX_MAX_ROWS;
	Width = maxlen*6+20;
	if (Width > MSG_BOX_MAX_WIDTH) Width = MSG_BOX_MAX_WIDTH;
	if (Width < MSG_BOX_MIN_WIDTH) Width = MSG_BOX_MIN_WIDTH;
	DlgHandle = DialogNewSimple (Width, Rows*10+40);
	if (DlgHandle == H_NULL) Ret = ER_MALLOC;
	if (Ret == ER_SUCCESS)
	{
		handle = DialogAddTitle (DlgHandle, Title, LeftButton, RightButton);
		if (handle == H_NULL) Ret = ER_MALLOC;
	}
	if (Ret == ER_SUCCESS)
	{
		ptr = Text;
		for (i=0; i<Rows && handle; i++)
		{
			handle = DialogAddText (DlgHandle, 10, i*10+20, ptr);
			ptr += strlen (ptr)+1;
			if (handle == H_NULL) Ret = ER_MALLOC;
		}
		if (Ret == ER_SUCCESS)
		{
			Ret = DialogDo (DlgHandle, CENTER, CENTER, NULL, NULL);
			if (Ret == KEY_ENTER) Ret = OK;
			if (Ret == KEY_ESC) Ret = CANCEL;
			stIdle();
			HeapFree (DlgHandle);
		}
	}

	if ( (Ret != OK) && (Ret != CANCEL) )
	{
		ShowFatalError(Ret);
		Ret = CANCEL;
	}

	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int RequestBox (char *Title, char *Request, char *Buffer, int BufSpace[], int RequestWidth, int Offset[], int num)
{
	HANDLE			DlgHandle;
	HANDLE			handle		= H_NULL;
	int				Ret			= ER_SUCCESS;
	char			*ptr		= Request;
	int				i;
	unsigned int	nMax		= 0;
	int				Width;
	
	for (i=0; i<num; i++)
	{
		if (strlen (ptr) > nMax) nMax = strlen (ptr);
		ptr += strlen (ptr) +1;
	}
	
	Width = nMax*6 + RequestWidth*6 + 30;
	if (Width < MSG_BOX_MIN_WIDTH) Width = MSG_BOX_MIN_WIDTH;
	DlgHandle = DialogNewSimple (Width, num*15 +35);
	if (DlgHandle == H_NULL) Ret = ER_MALLOC;
	else
	{
		handle = DialogAddTitle (DlgHandle, Title, BT_OK, BT_CANCEL);
		if (handle == H_NULL) Ret = ER_MALLOC;
		if (Ret == ER_SUCCESS)
		{
			ptr = Request;
			for (i=0; i<num && handle; i++)
			{
				handle = DialogAddRequest (DlgHandle, 10, i*15+20, ptr, Offset[i], BufSpace[i], RequestWidth);
				ptr += strlen (ptr) +1;
				if (handle == H_NULL) Ret = ER_MALLOC;
			}
		}
		if (Ret == ER_SUCCESS)
		{
			Ret = DialogDo (DlgHandle, CENTER, CENTER, Buffer, NULL);
			if (Ret == KEY_ENTER) Ret = OK;
			if (Ret == KEY_ESC) Ret = CANCEL;
		}
		HeapFree (DlgHandle);
	}
	
	if ( (Ret != OK) && (Ret != CANCEL) )
	{
		ShowFatalError (Ret);
		Ret = CANCEL;
	}
	
	stIdle();
	return (Ret);
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int WinInputStr(WINDOW *win, int X, int Y, char *buffer, unsigned short maxlen)
{
	short 			key;
  	char			DisplayBuffer[201];
  	char			MaskBuffer[201];
  	short			Attr;
  	int				Ret 					= OK;
  	unsigned short 	i 						= 0;
  	
	Attr = WinAttr (win, A_REPLACE);
  	buffer[0] = 0;
  	MaskBuffer[0] = 0;
	if (maxlen > 200) maxlen = 200;
	do
	{
		if (i < (maxlen)) sprintf (DisplayBuffer, "%s_ ", MaskBuffer);
		else sprintf (DisplayBuffer, "%s  ", MaskBuffer);
		WinStrXY (win, X, Y,  DisplayBuffer);
		key = ngetchx ();
		if (key >= ' ' && key <= '~' && i < maxlen)
		{
			MaskBuffer [i] = '*';
			buffer[i++] = key;
		}
		else if (key == KEY_BACKSPACE && i) i--;
	buffer[i] = 0;
	MaskBuffer[i] = 0;
	}
	while (key != KEY_ENTER && key != KEY_ESC);
	if (key == KEY_ESC)
	{
		buffer[0] = 0;
		Ret = CANCEL;
	}
	WinAttr (win, Attr);
	
	return Ret;
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int PasswordDlg (char *Title, char *Text, char *Password)
{
	WINDOW		*win;
	WIN_RECT	*winrect;
	int			x, y, dx;
	int			Ret			= ER_SUCCESS;
	
	memset (Password, 0, 9);
	x = LCD_WIDTH / 2;
	y = LCD_HEIGHT / 2;
	dx = strlen(Text)*6 /2 +10;
	if (dx < 90) dx = 90;	
	winrect = MakeWinRect (x-dx, y-35, x+dx, y+35);
	
	win = HeapAllocPtr (sizeof (WINDOW));
	if (win == NULL) Ret = ER_MALLOC; 

	if (Ret == ER_SUCCESS)
	{
		if (WinOpen (win, winrect, WF_SAVE_SCR | WF_TTY | WF_ROUNDEDBORDER | WF_TITLE, Title))
		{
			WinActivate (win);
			WinFont (win, F_6x8);
			WinStrXY (win, 10, 10, Text);
			DrawStaticButton (win, BT_OK, 10);
			DrawStaticButton (win, BT_CANCEL, dx*2 -87);
			stIdle();
			Ret = WinInputStr (win, 10, 25, Password, 8);
			WinClose (win);
		}
		else Ret = ER_MALLOC;
		HeapFreePtr (win);
	}
	
	return Ret; 
}

// --------------------------------------------------------------------------

// --------------------------------------------------------------------------
// --------------------------------------------------------------------------
// --------------------------------------------------------------------------

int	MakePopup (char *Title, char *Data, int Height, HANDLE *handle)
{
	char		*ptr	= Data;
	int			Ret		= ER_SUCCESS;
	
	*handle = PopupNew (Title, Height);
	if (*handle == H_NULL) Ret = ER_MALLOC;
	else
	{
		while ( (*ptr != '\0') && (Ret == ER_SUCCESS) )
		{
			if ( PopupAddText (*handle, -1, ptr, 0) == H_NULL)
			{
				Ret = ER_MALLOC;
				HeapFreeHNULL (handle);
			}
			ptr += strlen (ptr) +1;
		}
	}
	
	return Ret;
}

// --------------------------------------------------------------------------

int countDigits (int value)
{
	int		i, count = 0;
	
	for (i = 0; i < 16; i++)
	{
		if (value & 1) count++;
		value >>= 1;
	}
	
	return count;
}

void poke_w_ex (void *ptr, int value)
{
	*((char *)ptr) = (char)(value >> 8);
	*((char *)ptr+1) = (char)(value);
}


int getNumLetters (char *data, int width)
{
	int		n		= 0;
	int		wdt		= 0;
	int		flag	= 0;
	
	while (*data != 0 && !flag)
	{
		wdt += FontCharWidth (*data);
		if (wdt > width) flag = 1;
		else n++;
		data++;		
	}
	
	return n;
}

// --------------------------------------------------------------------------
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// oooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
// --------------------------------------------------------------------------
